@@ -6,7 +6,7 @@ from slugify import slugify
 from bs4 import Comment
 
 
-def subject_slug(subject, stopwords=['and', 'of']):
+def subject_slug(subject, stopwords=["and", "of"]):
     "create a slug for a subject value"
     if not subject:
         return subject
@@ -22,50 +22,58 @@ def first(x):
     except IndexError:
         return None
 
+
 def firstnn(x):
     "returns the first non-nil value within given iterable"
     return first(list(filter(None, x)))
 
+
 def strip_strings(value):
     def strip_string(value):
-        if hasattr(value, 'strip'):
+        if hasattr(value, "strip"):
             return value.strip()
         return value
+
     if type(value) == list:
         return list(map(strip_string, value))
     return strip_string(value)
 
+
 def strip_punctuation_space(value):
     "Strip excess whitespace prior to punctuation."
+
     def strip_punctuation(string):
         replacement_list = (
-            (' .',  '.'),
-            (' :',  ':'),
-            ('( ',  '('),
-            (' )',  ')'),
+            (" .", "."),
+            (" :", ":"),
+            ("( ", "("),
+            (" )", ")"),
         )
         for match, replacement in replacement_list:
             string = string.replace(match, replacement)
         return string
+
     if value == None:
         return None
     if type(value) == list:
         return [strip_punctuation(v) for v in value]
     return strip_punctuation(value)
 
-def join_sentences(string1, string2, glue='.'):
+
+def join_sentences(string1, string2, glue="."):
     "concatenate two sentences together with punctuation glue"
-    if not string1 or string1 == '':
+    if not string1 or string1 == "":
         return string2
-    if not string2 or string2 == '':
+    if not string2 or string2 == "":
         return string1
     # both are strings, continue joining them together with the glue and whitespace
     new_string = string1.rstrip()
     if not new_string.endswith(glue):
         new_string += glue
-    new_string += ' ' + string2.lstrip()
+    new_string += " " + string2.lstrip()
     return new_string
 
+
 def coerce_to_int(val, default=0xDEADBEEF):
     """Attempts to cast given value to an integer, return the original value if failed or the default if one provided."""
     try:
@@ -78,50 +86,61 @@ def coerce_to_int(val, default=0xDEADBEEF):
 
 def nullify(function):
     "Decorator. If empty list, returns None, else list."
+
     def wrapper(*args, **kwargs):
         value = function(*args, **kwargs)
-        if(type(value) == list and len(value) == 0):
+        if type(value) == list and len(value) == 0:
             return None
         return value
+
     return wrapper
 
+
 def strippen(function):
     "Decorator. Strip excess whitespace from return value."
+
     def wrapper(*args, **kwargs):
         return strip_strings(function(*args, **kwargs))
+
     return wrapper
 
+
 def inten(function):
     "Decorator. Attempts to convert return value to int"
+
     def wrapper(*args, **kwargs):
         return coerce_to_int(function(*args, **kwargs))
+
     return wrapper
 
+
 def clean_whitespace(value):
     if not value:
         return value
-    if hasattr(value, 'lstrip'):
-        value = value.lstrip('\n ')
-    if hasattr(value, 'rstrip'):
-        value = value.rstrip('\n ')
-    value = value.replace('\n', ' ')
+    if hasattr(value, "lstrip"):
+        value = value.lstrip("\n ")
+    if hasattr(value, "rstrip"):
+        value = value.rstrip("\n ")
+    value = value.replace("\n", " ")
     return value
 
-def date_struct(year, month, day, tz = "UTC"):
+
+def date_struct(year, month, day, tz="UTC"):
     """
     Given year, month and day numeric values and a timezone
     convert to structured date object
     """
     ymdtz = (year, month, day, tz)
     if None in ymdtz:
-        #logger.debug("a year, month, day or tz value was empty: %s" % str(ymdtz))
-        return None # return early if we have a bad value
+        # logger.debug("a year, month, day or tz value was empty: %s" % str(ymdtz))
+        return None  # return early if we have a bad value
     try:
-        return time.strptime("%s-%s-%s %s" % ymdtz,  "%Y-%m-%d %Z")
-    except(TypeError, ValueError):
-        #logger.debug("date failed to convert: %s" % str(ymdtz))
+        return time.strptime("%s-%s-%s %s" % ymdtz, "%Y-%m-%d %Z")
+    except (TypeError, ValueError):
+        # logger.debug("date failed to convert: %s" % str(ymdtz))
         pass
 
+
 def date_struct_nn(year, month, day, tz="UTC"):
     """
     Assemble a date object but if day or month is none set them to 1
@@ -133,6 +152,7 @@ def date_struct_nn(year, month, day, tz="UTC"):
         month = 1
     return date_struct(year, month, day, tz)
 
+
 def date_text(date_struct):
     # looks like: January 01, 2015
     return time.strftime("%B %d, %Y", date_struct) if date_struct else None
@@ -140,34 +160,41 @@ def date_text(date_struct):
 
 # TODO: if these are being converted to integers perhaps they shouldn't end with '_text' ?
 
+
 @inten
 def day_text(date_struct):
     return time.strftime("%d", date_struct) if date_struct else None
 
+
 @inten
 def month_text(date_struct):
     return time.strftime("%m", date_struct) if date_struct else None
 
+
 @inten
 def year_text(date_struct):
     return time.strftime("%Y", date_struct) if date_struct else None
 
+
 def date_timestamp(date_struct):
     try:
         return calendar.timegm(date_struct)
     except (TypeError, ValueError):
         pass
 
+
 def paragraphs(tags):
     "Given a list of tags, only return the paragraph tags"
     return list(filter(lambda tag: tag.name == "p", tags))
 
+
 def convert_testing_doi(doi):
     if doi is None:
         return doi
-    parts = doi.split('.')
+    parts = doi.split(".")
     new_msid = parts[-1][-5:]
-    return '.'.join(parts[0:-1] + [new_msid])
+    return ".".join(parts[0:-1] + [new_msid])
+
 
 def starts_with_doi(tag):
     if node_text(tag).strip().startswith("DOI:"):
@@ -175,46 +202,58 @@ def starts_with_doi(tag):
     else:
         return False
 
+
 def paragraph_is_only_doi(tag):
-    if (node_text(tag).strip().startswith('http://dx.doi.org')
-        and ' ' not in node_text(tag).strip()
-        and node_contents_str(tag).startswith('<ext-link ext-link-type="doi"')):
+    if (
+        node_text(tag).strip().startswith("http://dx.doi.org")
+        and " " not in node_text(tag).strip()
+        and node_contents_str(tag).startswith('<ext-link ext-link-type="doi"')
+    ):
         return True
     else:
         return False
 
+
 def doi_uri_to_doi(value):
     "Strip the uri schema from the start of DOI URL strings"
     if value is None:
         return value
-    replace_values = ['http://dx.doi.org/', 'https://dx.doi.org/',
-                      'http://doi.org/', 'https://doi.org/']
+    replace_values = [
+        "http://dx.doi.org/",
+        "https://dx.doi.org/",
+        "http://doi.org/",
+        "https://doi.org/",
+    ]
     for replace_value in replace_values:
-        value = value.replace(replace_value, '')
+        value = value.replace(replace_value, "")
     return value
 
+
 def doi_to_doi_uri(value):
     "Turn DOI into a valid uri"
     if value is None:
         return value
-    value = 'https://doi.org/' + value
+    value = "https://doi.org/" + value
     return value
 
+
 def remove_doi_paragraph(tags):
     "Given a list of tags, only return those whose text doesn't start with 'DOI:'"
     p_tags = list(filter(lambda tag: not starts_with_doi(tag), tags))
     p_tags = list(filter(lambda tag: not paragraph_is_only_doi(tag), p_tags))
     return p_tags
 
+
 def orcid_uri_to_orcid(value):
     "Strip the uri schema from the start of ORCID URL strings"
     if value is None:
         return value
-    replace_values = ['http://orcid.org/', 'https://orcid.org/']
+    replace_values = ["http://orcid.org/", "https://orcid.org/"]
     for replace_value in replace_values:
-        value = value.replace(replace_value, '')
+        value = value.replace(replace_value, "")
     return value
 
+
 def remove_tag_from_tag(tag, nodename):
     if not nodename:
         return tag
@@ -223,13 +262,14 @@ def remove_tag_from_tag(tag, nodename):
         unwanted_tag.decompose()
     return tag
 
+
 def component_acting_parent_tag(parent_tag, tag):
     """
     Only intended for use in getting components, look for tag name of fig-group
     and if so, find the first fig tag inside it as the acting parent tag
     """
     if parent_tag.name == "fig-group":
-        if (len(tag.find_previous_siblings("fig")) > 0):
+        if len(tag.find_previous_siblings("fig")) > 0:
             acting_parent_tag = first(extract_nodes(parent_tag, "fig"))
         else:
             # Do not return the first fig as parent of itself
@@ -238,11 +278,13 @@ def component_acting_parent_tag(parent_tag, tag):
         acting_parent_tag = parent_tag
     return acting_parent_tag
 
+
 #
 #
 #
 
-def extract_nodes(soup, nodename, attr = None, value = None):
+
+def extract_nodes(soup, nodename, attr=None, value=None):
     """
     Returns a list of tags (nodes) from the given soup matching the given nodename.
     If an optional attribute and value are given, these are used to filter the results
@@ -252,9 +294,11 @@ def extract_nodes(soup, nodename, attr = None, value = None):
         return list(filter(lambda tag: tag.get(attr) == value, tags))
     return list(tags)
 
+
 def node_text(tag):
     "Returns the text contents of a tag"
-    return getattr(tag, 'text', None)
+    return getattr(tag, "text", None)
+
 
 def node_contents_str(tag):
     """
@@ -263,14 +307,15 @@ def node_contents_str(tag):
     """
     if not tag:
         return None
-    tag_string = ''
+    tag_string = ""
     for child_tag in tag.children:
         if isinstance(child_tag, Comment):
             # BeautifulSoup does not preserve comment tags, add them back
-            tag_string += '<!--%s-->' % str(child_tag)
+            tag_string += "<!--%s-->" % str(child_tag)
         else:
             tag_string += str(child_tag)
-    return tag_string if tag_string != '' else None
+    return tag_string if tag_string != "" else None
+
 
 def first_parent(tag, nodename):
     """
@@ -283,6 +328,7 @@ def first_parent(tag, nodename):
         return first(list(filter(lambda tag: tag.name in nodename, tag.parents)))
     return None
 
+
 def tag_ordinal(tag):
     """
     Given a beautiful soup tag, look at the tags of the same name that come before it
@@ -292,16 +338,27 @@ def tag_ordinal(tag):
     tag_count = 0
     return len(tag.find_all_previous(tag.name)) + 1
 
+
 def tag_fig_ordinal(tag):
     """
     Meant for finding the position of fig tags with respect to whether
     they are for a main figure or a child figure
     """
     tag_count = 0
-    if 'specific-use' not in tag.attrs:
+    if "specific-use" not in tag.attrs:
         # Look for tags with no "specific-use" attribute
-        return len(list(filter(lambda tag: 'specific-use' not in tag.attrs,
-                          tag.find_all_previous(tag.name)))) + 1
+        return (
+            len(
+                list(
+                    filter(
+                        lambda tag: "specific-use" not in tag.attrs,
+                        tag.find_all_previous(tag.name),
+                    )
+                )
+            )
+            + 1
+        )
+
 
 def tag_sibling_ordinal(tag):
     """
@@ -327,21 +384,24 @@ def tag_limit_sibling_ordinal(tag, stop_tag_name):
 
     return tag_count
 
+
 def tag_subarticle_sibling_ordinal(tag):
-    return tag_limit_sibling_ordinal(tag, 'sub-article')
+    return tag_limit_sibling_ordinal(tag, "sub-article")
+
 
 def tag_appendix_sibling_ordinal(tag):
-    return tag_limit_sibling_ordinal(tag, 'app')
+    return tag_limit_sibling_ordinal(tag, "app")
+
 
 def tag_media_sibling_ordinal(tag):
     """
     Count sibling ordinal differently depending on if the
     mimetype is video or not
     """
-    if hasattr(tag, 'name') and tag.name != 'media':
+    if hasattr(tag, "name") and tag.name != "media":
         return None
 
-    nodenames = ['fig','supplementary-material','sub-article']
+    nodenames = ["fig", "supplementary-material", "sub-article"]
     first_parent_tag = first_parent(tag, nodenames)
 
     sibling_ordinal = None
@@ -350,17 +410,18 @@ def tag_media_sibling_ordinal(tag):
         # Start counting at 0
         sibling_ordinal = 0
         for media_tag in first_parent_tag.find_all(tag.name):
-            if 'mimetype' in tag.attrs and tag['mimetype'] == 'video':
+            if "mimetype" in tag.attrs and tag["mimetype"] == "video":
                 # Count all video type media tags
-                if 'mimetype' in media_tag.attrs and tag['mimetype'] == 'video':
+                if "mimetype" in media_tag.attrs and tag["mimetype"] == "video":
                     sibling_ordinal += 1
                 if media_tag == tag:
                     break
 
             else:
                 # Count all non-video type media tags
-                if (('mimetype' not in media_tag.attrs)
-                    or ('mimetype' in media_tag.attrs and tag['mimetype'] != 'video')):
+                if ("mimetype" not in media_tag.attrs) or (
+                    "mimetype" in media_tag.attrs and tag["mimetype"] != "video"
+                ):
                     sibling_ordinal += 1
                 if media_tag == tag:
                     break
@@ -369,16 +430,23 @@ def tag_media_sibling_ordinal(tag):
         sibling_ordinal = 1
         for prev_tag in tag.find_all_previous(tag.name):
             if not first_parent(prev_tag, nodenames):
-                if 'mimetype' in tag.attrs and tag['mimetype'] == 'video':
+                if "mimetype" in tag.attrs and tag["mimetype"] == "video":
                     # Count all video type media tags
-                    if supp_asset(prev_tag) == supp_asset(tag) and 'mimetype' in prev_tag.attrs:
+                    if (
+                        supp_asset(prev_tag) == supp_asset(tag)
+                        and "mimetype" in prev_tag.attrs
+                    ):
                         sibling_ordinal += 1
                 else:
-                    if supp_asset(prev_tag) == supp_asset(tag) and 'mimetype' not in prev_tag.attrs:
+                    if (
+                        supp_asset(prev_tag) == supp_asset(tag)
+                        and "mimetype" not in prev_tag.attrs
+                    ):
                         sibling_ordinal += 1
 
     return sibling_ordinal
 
+
 def tag_supplementary_material_sibling_ordinal(tag):
     """
     Strategy is to count the previous supplementary-material tags
@@ -386,10 +454,10 @@ def tag_supplementary_material_sibling_ordinal(tag):
     The result is its position inside any parent tag that
     are the same asset type
     """
-    if hasattr(tag, 'name') and tag.name != 'supplementary-material':
+    if hasattr(tag, "name") and tag.name != "supplementary-material":
         return None
 
-    nodenames = ['fig','media','sub-article']
+    nodenames = ["fig", "media", "sub-article"]
     first_parent_tag = first_parent(tag, nodenames)
 
     sibling_ordinal = 1
@@ -422,16 +490,17 @@ def supp_asset(tag):
     has depending on how many of each type is present
     """
     # Default
-    asset = 'supp'
+    asset = "supp"
     if first(extract_nodes(tag, "label")):
         label_text = node_text(first(extract_nodes(tag, "label"))).lower()
         # Keyword match the label
-        if label_text.find('code') > 0:
-            asset = 'code'
-        elif label_text.find('data') > 0:
-            asset = 'data'
+        if label_text.find("code") > 0:
+            asset = "code"
+        elif label_text.find("data") > 0:
+            asset = "data"
     return asset
 
+
 def copy_attribute(source, source_key, destination, destination_key=None):
     if destination_key is None:
         destination_key = source_key
@@ -439,13 +508,18 @@ def copy_attribute(source, source_key, destination, destination_key=None):
         if source is not None and destination is not None and source_key in source:
             destination[destination_key] = source[source_key]
 
-def first_node_str_contents(soup, nodename, attr = None, value = None):
-    return node_contents_str(first(extract_nodes(soup, nodename, attr=attr, value=value)))
+
+def first_node_str_contents(soup, nodename, attr=None, value=None):
+    return node_contents_str(
+        first(extract_nodes(soup, nodename, attr=attr, value=value))
+    )
+
 
 def set_if_value(dictionary, key, value):
     if value is not None:
         dictionary[key] = value
 
+
 def prune_dict_of_none_values(dictionary):
     return dict((k, v) for k, v in dictionary.items() if v is not None)
 
@@ -470,11 +544,13 @@ def text_to_title(value):
             title = title.rstrip(" .:")
     return title
 
+
 def rstrip_punctuation(value):
     "strip punctuation from the end of a label or title"
     if not value:
         return value
-    return value.rstrip('.:')
+    return value.rstrip(".:")
+
 
 def escape_unmatched_angle_brackets(string, allowed_tag_fragments=()):
     """
@@ -488,30 +564,35 @@ def escape_unmatched_angle_brackets(string, allowed_tag_fragments=()):
         return string
 
     # Split string on tags
-    tags = re.split('(<.*?>)', string)
-    #print tags
+    tags = re.split("(<.*?>)", string)
+    # print tags
 
     for i, val in enumerate(tags):
         # Use angle bracket character counts to find unmatched tags
         #  as well as our allowed_tags list to ignore good tags
 
-        if val.count('<') == val.count('>') and not val.startswith(allowed_tag_fragments):
-            val = val.replace('<', '&lt;')
-            val = val.replace('>', '&gt;')
+        if val.count("<") == val.count(">") and not val.startswith(
+            allowed_tag_fragments
+        ):
+            val = val.replace("<", "&lt;")
+            val = val.replace(">", "&gt;")
         else:
             # Count how many unmatched tags we have
-            while val.count('<') != val.count('>'):
-                if val.count('<') != val.count('>') and val.count('<') > 0:
-                    val = val.replace('<', '&lt;', 1)
-                elif val.count('<') != val.count('>') and val.count('>') > 0:
-                    val = val.replace('>', '&gt;', 1)
-            if val.count('<') == val.count('>') and not val.startswith(allowed_tag_fragments):
+            while val.count("<") != val.count(">"):
+                if val.count("<") != val.count(">") and val.count("<") > 0:
+                    val = val.replace("<", "&lt;", 1)
+                elif val.count("<") != val.count(">") and val.count(">") > 0:
+                    val = val.replace(">", "&gt;", 1)
+            if val.count("<") == val.count(">") and not val.startswith(
+                allowed_tag_fragments
+            ):
                 # Send it through again in case there are nested unmatched tags
                 val = escape_unmatched_angle_brackets(val, allowed_tag_fragments)
 
         tags[i] = val
 
-    return ''.join(tags)
+    return "".join(tags)
+
 
 def escape_ampersand(string):
     """
@@ -522,40 +603,50 @@ def escape_ampersand(string):
         return string
     start_with_match = r"(\#x(....);|lt;|gt;|amp;)"
     # The pattern below is match & that is not immediately followed by #
-    string = re.sub(r"&(?!" + start_with_match + ")", '&amp;', string)
+    string = re.sub(r"&(?!" + start_with_match + ")", "&amp;", string)
     return string
 
 
 def remove_tag(tag_name, string):
     "remove open tag and its attributes and remove close tag from an XML string"
-    pattern = r'\n*</?%s.*?>\n*' % tag_name
-    return re.sub(pattern, '', string) if string else ''
+    pattern = r"\n*</?%s.*?>\n*" % tag_name
+    return re.sub(pattern, "", string) if string else ""
 
 
 def remove_tag_and_text(tag_name, string):
     "remove open tag and close tag and the contents between them from an XML string"
-    pattern = r'\n*<%s.*?>.*?</%s>\n*' % (tag_name, tag_name)
-    return remove_tag(tag_name, re.sub(pattern, '', string)) if string else ''
+    pattern = r"\n*<%s.*?>.*?</%s>\n*" % (tag_name, tag_name)
+    return remove_tag(tag_name, re.sub(pattern, "", string)) if string else ""
 
 
 def references_author_person(ref_author):
     author_json = OrderedDict()
     author_json["type"] = "person"
     author_name = OrderedDict()
-    author_name["preferred"] = author_preferred_name(ref_author.get("surname"),
-                                                     ref_author.get("given-names"),
-                                                     ref_author.get("suffix"))
-    author_name["index"] = author_index_name(ref_author.get("surname"),
-                                             ref_author.get("given-names"),
-                                             ref_author.get("suffix"))
+    author_name["preferred"] = author_preferred_name(
+        ref_author.get("surname"),
+        ref_author.get("given-names"),
+        ref_author.get("suffix"),
+    )
+    author_name["index"] = author_index_name(
+        ref_author.get("surname"),
+        ref_author.get("given-names"),
+        ref_author.get("suffix"),
+    )
     author_json["name"] = author_name
     return author_json
 
+
 def author_preferred_name(surname, given_names, suffix):
-    return " ".join([element for element in [given_names, surname, suffix] if element is not None])
+    return " ".join(
+        [element for element in [given_names, surname, suffix] if element is not None]
+    )
+
 
 def author_index_name(surname, given_names, suffix):
-    index_name = ", ".join([element for element in [surname, given_names, suffix] if element is not None])
+    index_name = ", ".join(
+        [element for element in [surname, given_names, suffix] if element is not None]
+    )
     return index_name
 
 
